@@ -2,7 +2,7 @@
 /// @file  api_c.cpp
 ///        primecount's C API.
 ///
-/// Copyright (C) 2020 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2025 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -32,6 +32,32 @@ int64_t primecount_pi(int64_t x)
   }
 }
 
+pc_int128_t primecount_pi_128(pc_int128_t x)
+{
+  try
+  {
+    primecount::pc_int128_t n;
+    n.lo = x.lo;
+    n.hi = x.hi;
+
+    auto pix = primecount::pi(n);
+
+    pc_int128_t res;
+    res.lo = pix.lo;
+    res.hi = pix.hi;
+    return res;
+  }
+  catch(const std::exception& e)
+  {
+    std::cerr << "primecount_pi_128: " << e.what() << std::endl;
+
+    pc_int128_t res;
+    res.lo = ~0ull;
+    res.hi = -1;
+    return res;
+  }
+}
+
 int primecount_pi_str(const char* x, char* res, size_t len)
 {
   try
@@ -84,6 +110,32 @@ int64_t primecount_nth_prime(int64_t n)
   }
 }
 
+pc_int128_t primecount_nth_prime_128(pc_int128_t n)
+{
+  try
+  {
+    primecount::pc_int128_t x;
+    x.lo = n.lo;
+    x.hi = n.hi;
+
+    auto nth_prime = primecount::nth_prime(x);
+
+    pc_int128_t res;
+    res.lo = nth_prime.lo;
+    res.hi = nth_prime.hi;
+    return res;
+  }
+  catch(const std::exception& e)
+  {
+    std::cerr << "primecount_nth_prime_128: " << e.what() << std::endl;
+
+    pc_int128_t res;
+    res.lo = ~0ull;
+    res.hi = -1;
+    return res;
+  }
+}
+
 int64_t primecount_phi(int64_t x, int64_t a)
 {
   try
