@@ -94,6 +94,7 @@ T S2_hard_thread(T x,
     // For b < min_b there are no special leaves:
     // low <= x / (primes[b] * m) < high
     sieve.pre_sieve(primes, min_b - 1, low, high);
+    sieve.init_counter(low, high);
     int64_t b = min_b;
 
     // For c + 1 <= b <= pi_sqrty
@@ -245,7 +246,7 @@ int64_t S2_hard_default(int64_t x,
   {
     print("");
     print("=== S2_hard(x, y) ===");
-    print("Algorithm: " SIEVE_COUNT_ALGO_NAME);
+    print("Algorithm: " DEFAULT_SIEVE_COUNT_ALGO_NAME);
     print_vars(x, y, c, threads);
     time = get_time();
   }
@@ -298,7 +299,7 @@ int128_t S2_hard_default(int128_t x,
   {
     print("");
     print("=== S2_hard(x, y) ===");
-    print("Algorithm: " SIEVE_COUNT_ALGO_NAME);
+    print("Algorithm: " DEFAULT_SIEVE_COUNT_ALGO_NAME);
     print_vars(x, y, c, threads);
     time = get_time();
   }
