@@ -25,16 +25,17 @@
 ///        In-depth description of this algorithm:
 ///        https://github.com/kimwalisch/primecount/blob/master/doc/Easy-Special-Leaves.md
 ///
-/// Copyright (C) 2024 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2025 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
 ///
 
+#include "LoadBalancerAC.hpp"
+#include "SegmentedPiTable.hpp"
+
 #include <PiTable.hpp>
-#include <SegmentedPiTable.hpp>
 #include <primecount-internal.hpp>
-#include <LoadBalancerAC.hpp>
 #include <fast_div.hpp>
 #include <generate_primes.hpp>
 #include <gourdon.hpp>
