@@ -45,7 +45,7 @@ T B_thread(T x,
   int64_t sqrtx = isqrt(x);
   int64_t start = max(y, min(x / high, sqrtx));
   int64_t stop = min(x / low, sqrtx);
-  primesieve::iterator it1(stop + 1, start);
+  primesieve::iterator it1(stop, start);
   int64_t prime = it1.prev_prime();
 
   if (prime <= start)
@@ -61,7 +61,7 @@ T B_thread(T x,
 
   // All other iterations compute pi(x / prime)
   // using a prime sieve.
-  primesieve::iterator it2(xp, high);
+  primesieve::iterator it2(xp + 1, high);
   it2.generate_next_primes();
 
   // \sum_{i = pi[start]+1}^{pi[stop]} pi(x / primes[i])
@@ -118,14 +118,16 @@ int64_t B(int64_t x,
           int threads,
           bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== B(x, y) ===");
     print_gourdon_vars(x, y, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   int64_t sum = B_OpenMP((uint64_t) x, y, threads, is_print);
 
   if (is_print)
@@ -141,14 +143,16 @@ int128_t B(int128_t x,
            int threads,
            bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== B(x, y) ===");
     print_gourdon_vars(x, y, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   int128_t sum = B_OpenMP((uint128_t) x, y, threads, is_print);
 
   if (is_print)
